/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx;

import com.sonar.cxx.sslr.api.Grammar;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.sonar.api.batch.rule.CheckFactory;
import org.sonar.api.batch.rule.internal.ActiveRulesBuilder;
import org.sonar.api.batch.rule.internal.NewActiveRule;
import org.sonar.api.config.internal.MapSettings;
import org.sonar.api.resources.Language;
import org.sonar.api.rule.RuleKey;
import org.sonar.api.server.rule.RulesDefinition;
import org.sonar.check.Rule;
import org.sonar.cxx.squidbridge.SquidAstVisitor;
import org.sonar.cxx.squidbridge.checks.SquidCheck;

import javax.annotation.CheckForNull;
import java.util.ArrayList;
import java.util.Collections;

import static org.assertj.core.api.Assertions.assertThat;

class CxxChecksTest {

  private static final String DEFAULT_REPOSITORY_KEY = "DefaultRuleRepository";
  private static final String DEFAULT_RULE_KEY = "MyRule";
  private static final String CUSTOM_REPOSITORY_KEY = "CustomRuleRepository";
  private static final String CUSTOM_RULE_KEY = "MyCustomRule";

  private MyCustomPlSqlRulesDefinition customRulesDefinition;
  private CheckFactory checkFactory;

  @BeforeEach
  public void setUp() {
    var activeRules = new ActiveRulesBuilder()
      .addRule(new NewActiveRule.Builder()
        .setRuleKey(RuleKey.of(DEFAULT_REPOSITORY_KEY, DEFAULT_RULE_KEY))
        .build())
      .addRule(new NewActiveRule.Builder()
        .setRuleKey(RuleKey.of(CUSTOM_REPOSITORY_KEY, CUSTOM_RULE_KEY))
        .build())
      .build();
    checkFactory = new CheckFactory(activeRules);

    customRulesDefinition = new MyCustomPlSqlRulesDefinition();
    var context = new RulesDefinition.Context();
    customRulesDefinition.define(context);
  }

  @SuppressWarnings("rawtypes")
  @Test
  void shouldReturnDefaultChecks() {
    var checks = CxxChecks.createCxxCheck(checkFactory);
    checks.addChecks(DEFAULT_REPOSITORY_KEY, new ArrayList<>(Collections.singletonList(MyRule.class)));

    SquidAstVisitor<Grammar> defaultCheck = check(checks, DEFAULT_REPOSITORY_KEY, DEFAULT_RULE_KEY);

    assertThat(checks.all()).hasSize(1);
    assertThat(checks.ruleKey(defaultCheck)).isNotNull();
    assertThat(checks.ruleKey(defaultCheck).rule()).isEqualTo(DEFAULT_RULE_KEY);
    assertThat(checks.ruleKey(defaultCheck).repository()).isEqualTo(DEFAULT_REPOSITORY_KEY);
  }

  @Test
  void shouldReturnCustomChecks() {
    var checks = CxxChecks.createCxxCheck(checkFactory);
    checks.addCustomChecks(new CustomCxxRulesDefinition[]{customRulesDefinition});

    SquidAstVisitor<Grammar> customCheck = check(checks, CUSTOM_REPOSITORY_KEY, CUSTOM_RULE_KEY);

    assertThat(checks.all()).hasSize(1);
    assertThat(checks.ruleKey(customCheck)).isNotNull();
    assertThat(checks.ruleKey(customCheck).rule()).isEqualTo(CUSTOM_RULE_KEY);
    assertThat(checks.ruleKey(customCheck).repository()).isEqualTo(CUSTOM_REPOSITORY_KEY);
  }

  @Test
  void shouldWorkWithoutCustomChecks() {
    var checks = CxxChecks.createCxxCheck(checkFactory);
    checks.addCustomChecks(null);
    assertThat(checks.all()).isEmpty();
  }

  @SuppressWarnings("rawtypes")
  @Test
  void shouldNotReturnRuleKeyIfCheckDoesNotExists() {
    var checks = CxxChecks.createCxxCheck(checkFactory);
    checks.addChecks(DEFAULT_REPOSITORY_KEY, new ArrayList<>(Collections.singletonList(MyRule.class)));
    assertThat(checks.ruleKey(new MyCustomRule())).isNull();
  }

  @CheckForNull
  public SquidAstVisitor<Grammar> check(CxxChecks cxxChecks, String repository, String rule) {
    RuleKey key = RuleKey.of(repository, rule);

    SquidAstVisitor<Grammar> check;
    for (var checks : cxxChecks.getChecks()) {
      check = checks.of(key);

      if (check != null) {
        return check;
      }
    }
    return null;
  }

  @Rule(key = DEFAULT_RULE_KEY, name = "This is the default rule", description = "desc")
  public static class MyRule extends SquidCheck<Grammar> {
  }

  @Rule(key = CUSTOM_RULE_KEY, name = "This is the custom rule", description = "desc")
  public static class MyCustomRule extends SquidCheck<Grammar> {
  }

  public static class MyCustomPlSqlRulesDefinition extends CustomCxxRulesDefinition {

    private static final Language LANGUAGE = new CxxLanguage(new MapSettings().asConfig());

    @Override
    public String repositoryName() {
      return "Custom Rule Repository";
    }

    @Override
    public String repositoryKey() {
      return CUSTOM_REPOSITORY_KEY;
    }

    @SuppressWarnings("rawtypes")
    @Override
    public Class[] checkClasses() {
      return new Class[]{MyCustomRule.class};
    }

    @Override
    public Language getLanguage() {
      return LANGUAGE;
    }
  }

}
